# Copyright (c) 2017, Michael Boyle
# See LICENSE file for details: <https://github.com/moble/quaternion/blob/master/LICENSE>

from __future__ import division, print_function, absolute_import



def mean_rotor_in_chordal_metric(R, t=None):
    """Return rotor that is closest to all R in the least-squares sense

    This can be done (quasi-)analytically because of the simplicity of
    the chordal metric function.  It is assumed that the input R values
    all are normalized (or at least have the same norm).

    Note that the `t` argument is optional.  If it is present, the
    times are used to weight the corresponding integral.  If it is not
    present, a simple sum is used instead (which may be slightly
    faster).  However, because a spline is used to do this integral,
    the number of input points must be at least 4 (one more than the
    degree of the spline).

    """
    import numpy as np
    from . import as_float_array
    from .calculus import definite_integral
    if t is None:
        return np.sum(R).normalized()
    if len(t) < 4 or len(R) < 4:
        raise ValueError('Input arguments must have length greater than 3; their lengths are {0} and {1}.'.format(len(R), len(t)))
    mean = definite_integral(as_float_array(R), t)
    return np.quaternion(*mean).normalized()


def optimal_alignment_in_chordal_metric(Ra, Rb, t=None):
    """Return Rd such that Rd*Rb is as close to Ra as possible

    This function simply encapsulates the mean rotor of Ra/Rb.

    As in the `mean_rotor_in_chordal_metric` function, the `t`
    argument is optional.  If it is present, the times are used to
    weight the corresponding integral.  If it is not present, a simple
    sum is used instead (which may be slightly faster).

    """
    return mean_rotor_in_chordal_metric(Ra / Rb, t)


def mean_rotor_in_intrinsic_metric(R, t=None):
    raise NotImplementedError()



#!/usr/bin/env python

from __future__ import print_function, division, absolute_import

import math
import numpy as np
import quaternion
from numpy import *
import pytest

try:
    import scipy
    has_scipy = True
except:
    has_scipy = False

eps = np.finfo(float).eps


@pytest.mark.skipif(not has_scipy, reason="Scipy is not installed")
def test_subset_interpolation():
    from quaternion.calculus import spline
    t = np.linspace(0, 10, 100)
    f = np.sin(t)

    for i1, i2 in [[0, 100], [0, -1], [10, -10], [11, -10], [10, -11], [11, -11], [21, -21], [31, -31]]:
        f_out = spline(f, t, t_out=t[i1:i2])
        f_sub = f[i1:i2]
        assert np.allclose(f_out, f_sub, atol=2*eps, rtol=2*eps)


@pytest.mark.skipif(not has_scipy, reason="Scipy is not installed")
def test_differentiation():
    from quaternion.calculus import spline
    t = np.linspace(0, 10, 1000)

    f = np.sin(t)
    fprime = np.cos(t)
    assert np.allclose(spline(f, t, spline_degree=5, derivative_order=1), fprime, atol=1e-11, rtol=1e-11)

    f = np.exp(1j*t)
    fprime = 1j*np.exp(1j*t)
    assert np.allclose(spline(f, t, spline_degree=5, derivative_order=1), fprime, atol=1e-11, rtol=1e-11)


@pytest.mark.skipif(not has_scipy, reason="Scipy is not installed")
def test_antiderivative():
    from quaternion.calculus import spline
    t = np.linspace(0, 10, 1000)

    f = np.cos(t)
    fint = np.sin(t)
    assert np.allclose(spline(f, t, spline_degree=5, derivative_order=-1), fint, atol=1e-11, rtol=1e-11)

    f = np.exp(1j*t)
    fint = 1j*(1-np.exp(1j*t))
    assert np.allclose(spline(f, t, spline_degree=5, derivative_order=-1), fint, atol=1e-11, rtol=1e-11)


@pytest.mark.skipif(not has_scipy, reason="Scipy is not installed")
def test_integral():
    from quaternion.calculus import spline
    t = np.linspace(0, 10, 1000)

    f = np.cos(t)
    fint = np.sin(t)
    assert np.allclose(spline(f, t, spline_degree=5, definite_integral_bounds=(t[0], t[-1])),
                       fint[-1]-fint[0], atol=1e-11, rtol=1e-11)

    f = np.exp(1j*t)
    fint = 1j*(1-np.exp(1j*t))
    # print(max(abs(spline(f, t, spline_degree=5, derivative_order=-1)-fint)))
    assert np.allclose(spline(f, t, spline_degree=5, definite_integral_bounds=(t[0], t[-1])),
                       fint[-1]-fint[0], atol=1e-11, rtol=1e-11)
